% clear all; clc;
% 
% %                  Specimen - parameters
% % h - longitudinal size; b,h - crossectional size
% h = 200; % [mm]
% b = 20; t = 10; % [mm]
% 
% %                  Fibres - parameters
% % Geometry
% Lf = [12; 0]; % [mm]
% df = [0.04; 0]; % [mm]
% % Material
% Ef = [20000; 2030]; % [MPa]
% ftf = [1300; 100]; % [MPa]
% Vff = 0.02; % volume fraction of fiber

function f = GenFibSaw(b, t, h, Lf, df, Vff)

% function generates fibres in larger volume
% fiber are sawed along the sides of specimen
% computation is parallel
%
% in:   b, t, h - dimension of specimen
%       Lf - length of fiber
%       df - diameter of fiber
%       Vff - volume fraction of fiber
% out:  f - table of fibers
%       Column 1 - initial x [mm]
%       Column 2 - initial  y [mm]
%       Column 3 - initial  z [mm]
%       Column 4 - end x [mm]
%       Column 5 - end y [mm]
%       Column 6 - end z [mm]
%       Column 7 - angle thau (angle from cross plane) [rad]
%       Column 8 - directional angle [rad]

Vf=pi*df(1,1)^2/4*Lf(1,1); % [mm3]
V=(b+Lf(1,1))*(t+Lf(1,1))*(h+Lf(1,1)); % [mm3]
n=round((V*Vff)/Vf);		% number of fibers [-]

fal=zeros(n,8);

parfor i=1:n
	X=random('Uniform',(0-Lf(1,1)/2),(b+Lf(1,1)/2)); Y=random('Uniform',(0-Lf(1,1)/2),(t+Lf(1,1)/2)); Z=random('Uniform',(0-Lf(1,1)/2),(h+Lf(1,1)/2));
	fi=random('Uniform',0,2*pi); theta=random('Uniform',-pi/2,pi/2);
	Lfc = random('Normal',Lf(1,1),Lf(2,1)); % length of current fiber
	
	Xp=X-cos(fi)*cos(theta)*Lfc/2; Yp=Y-sin(fi)*cos(theta)*Lfc/2; Zp=Z-sin(theta)*Lfc/2;
	Xk=X+cos(fi)*cos(theta)*Lfc/2; Yk=Y+sin(fi)*cos(theta)*Lfc/2; Zk=Z+sin(theta)*Lfc/2;
	
	Lff=Lfc;
	m=0;
	if((0-Xp)/(cos(fi)*cos(theta))<=Lfc && (0-Xp)/(cos(fi)*cos(theta))>0 && (Yp+(0-Xp)/(cos(fi)*cos(theta))*sin(fi)*cos(theta))>=0 && (Yp+(0-Xp)/(cos(fi)*cos(theta))*sin(fi)*cos(theta))<=t && (Zp+(0-Xp)/(cos(fi)*cos(theta))*sin(theta))>=0 && (Zp+(0-Xp)/(cos(fi)*cos(theta))*sin(theta))<=h)
		m=m+1;
	end
	if((b-Xp)/(cos(fi)*cos(theta))<=Lfc && (b-Xp)/(cos(fi)*cos(theta))>0 && (Yp+(b-Xp)/(cos(fi)*cos(theta))*sin(fi)*cos(theta))>=0 && (Yp+(b-Xp)/(cos(fi)*cos(theta))*sin(fi)*cos(theta))<=t && (Zp+(b-Xp)/(cos(fi)*cos(theta))*sin(theta))>=0 && (Zp+(b-Xp)/(cos(fi)*cos(theta))*sin(theta))<=h)
		m=m+1;
	end
	if((0-Yp)/(sin(fi)*cos(theta))<=Lfc && (0-Yp)/(sin(fi)*cos(theta))>0 && (Xp+(0-Yp)/(sin(fi)*cos(theta))*cos(fi)*cos(theta))>=0 && (Xp+(0-Yp)/(sin(fi)*cos(theta))*cos(fi)*cos(theta))<=b && (Zp+(0-Yp)/(sin(fi)*cos(theta))*sin(theta))>=0 && (Zp+(0-Yp)/(sin(fi)*cos(theta))*sin(theta))<=h)
		m=m+1;
	end
	if((t-Yp)/(sin(fi)*cos(theta))<=Lfc && (t-Yp)/(sin(fi)*cos(theta))>0 && (Xp+(t-Yp)/(sin(fi)*cos(theta))*cos(fi)*cos(theta))>=0 && (Xp+(t-Yp)/(sin(fi)*cos(theta))*cos(fi)*cos(theta))<=b && (Zp+(t-Yp)/(sin(fi)*cos(theta))*sin(theta))>=0 && (Zp+(t-Yp)/(sin(fi)*cos(theta))*sin(theta))<=h)
		m=m+1;
	end
	if((0-Zp)/sin(theta)<=Lfc && (0-Zp)/sin(theta)>0 && (Xp+((0-Zp)/sin(theta))*cos(fi)*cos(theta))>=0 && (Xp+((0-Zp)/sin(theta))*cos(fi)*cos(theta))<=b && (Yp+((0-Zp)/sin(theta))*sin(fi)*cos(theta))>=0 && (Yp+((0-Zp)/sin(theta))*sin(fi)*cos(theta))<=t)
		m=m+1;
	end
	if((h-Zp)/sin(theta)<=Lfc && (h-Zp)/sin(theta)>0 && (Xp+((h-Zp)/sin(theta))*cos(fi)*cos(theta))>=0 && (Xp+((h-Zp)/sin(theta))*cos(fi)*cos(theta))<=b && (Yp+((h-Zp)/sin(theta))*sin(fi)*cos(theta))>=0 && (Yp+((h-Zp)/sin(theta))*sin(fi)*cos(theta))<=t)
		m=m+1;
	end
	if(Xp>=0 && Xp<=b && Yp>=0 && Yp<=t && Zp>=0 && Zp<=h && Xk>=0 && Xk<=b && Yk>=0 && Yk<=t && Zk>=0 && Zk<=h)
		fal(i,:) = [Xp, Yp, Zp, Xk, Yk, Zk, theta, fi];
	elseif(m==0)
	else
		if ((Xp<0) || (Xp>b) || (Yp<0) || (Yp>t) || (Zp<0) || (Zp>h))
			if(Xp<0)
				Xp=0; Lff=(Xk-Xp)/(cos(fi)*cos(theta));
				Yp=Yk-sin(fi)*cos(theta)*Lff; Zp=Zk-sin(theta)*Lff;
			end	
			if(Xp>b)
				Xp=b; Lff=(Xk-Xp)/(cos(fi)*cos(theta));
				Yp=Yk-sin(fi)*cos(theta)*Lff; Zp=Zk-sin(theta)*Lff;
			end
			if(Yp<0)
				Yp=0; Lff=(Yk-Yp)/(sin(fi)*cos(theta));
				Xp=Xk-cos(fi)*cos(theta)*Lff; Zp=Zk-sin(theta)*Lff;
			end
			if(Yp>t)
				Yp=t; Lff=(Yk-Yp)/(sin(fi)*cos(theta));
				Xp=Xk-cos(fi)*cos(theta)*Lff; Zp=Zk-sin(theta)*Lff;
			end
			if(Zp<0)
				Zp=0; Lff=(Zk-Zp)/sin(theta);
				Xp=Xk-cos(fi)*cos(theta)*Lff; Yp=Yk-sin(fi)*cos(theta)*Lff;
			end
			if(Zp>h)
				Zp=h; Lff=(Zk-Zp)/sin(theta);
				Xp=Xk-cos(fi)*cos(theta)*Lff; Yp=Yk-sin(fi)*cos(theta)*Lff;
			end
		end
		if ((Xk<0) || (Xk>b) || (Yk<0) || (Yk>t) || (Zk<0) || (Zk>h))
			if(Xk<0)
				Xk=0; Lff=(Xk-Xp)/(cos(fi)*cos(theta));
				Yk=Yp+sin(fi)*cos(theta)*Lff; Zk=Zp+sin(theta)*Lff;
			end	
			if(Xk>b)
				Xk=b; Lff=(Xk-Xp)/(cos(fi)*cos(theta));
				Yk=Yp+sin(fi)*cos(theta)*Lff; Zk=Zp+sin(theta)*Lff;
			end
			if(Yk<0)
				Yk=0; Lff=(Yk-Yp)/(sin(fi)*cos(theta));
				Xk=Xp+cos(fi)*cos(theta)*Lff; Zk=Zp+sin(theta)*Lff;
			end
			if(Yk>t)
				Yk=t; Lff=(Yk-Yp)/(sin(fi)*cos(theta));
				Xk=Xp+cos(fi)*cos(theta)*Lff; Zk=Zp+sin(theta)*Lff;
			end
			if(Zk<0)
				Zk=0; Lff=(Zk-Zp)/sin(theta);
				Xk=Xp+cos(fi)*cos(theta)*Lff; Yk=Yp+sin(fi)*cos(theta)*Lff;
			end
			if(Zk>h)
				Zk=h; Lff=(Zk-Zp)/sin(theta);
				Xk=Xp+cos(fi)*cos(theta)*Lff; Yk=Yp+sin(fi)*cos(theta)*Lff;
			end
		end
		fal(i,:) = [Xp, Yp, Zp, Xk, Yk, Zk, theta, fi];
	end
end
NonZero = [nnz(fal(:,1)), nnz(fal(:,2)), nnz(fal(:,3)), nnz(fal(:,4)), nnz(fal(:,5)), nnz(fal(:,6)), nnz(fal(:,7)), nnz(fal(:,8))];
CooCol = 1;
 while( NonZero(CooCol) ~= max(NonZero) )
	 CooCol = CooCol+1;
 end
 CooRow = find(fal(:,CooCol));
 f=fal(CooRow, :);