function results = Program(input)

%%                                               Fiber Bridging
%
% This program computes cohesion law (stress-displacement relationship
% for random generated fibres in a given prism specimen
%
% (c) Michal Prinosil 2012

%%                                               Input
%                                               Input data
textControl = getappdata(0,'textControl');
set(textControl, 'string', ''); pause(0.5);

%                  Fibres - parameters
% Geometry
Lf = input.Lf;
df = input.df;
% Material
Ef = input.Ef;
ftf = input.ftf;
Vff = input.Vff;

%                  Matrix - parameters
Em = input.Em;

%                  Interface - parameters
tau0 = input.tau0;
Gd = input.Gd;
beta = input.beta;

%                  Other parameters
sc = input.sc;
src = input.src;
sra = input.sra;
ftf(1,1) = ftf(1,1) * sra;

%                  Specimen - parameters
numCrack = input.numCrack;

%                                               The calculation parameters

%                  Parallelization

numCor = input.numCor;

%                  Generating fibers
% Parameter: typeGen
%        0 - loading input file
%                  FileName - string - name of file for loading
%        1 - generating fibers in given volume - cast
%        2 - generating fibers in larger volume and saw along sides - saw

typeGen = 2;
% FileName =  'FibGeo.txt';

%                  Calculation parameters
% Parameter: typeCalc
%        0 - single way pull - out
%        1 - two way pull - out
typeCalc = 1;
dc = 0.01; % increment of crack opening
cc = 0.0001; % convergence criterion

%%                                               Other input parameters
eta = (Ef(1,1)*Vff)/(Em(1,1)*(1-Vff));
%                  Specimen - parameters
% b,h - crossectional size
if input.genSpecGeo == 0	 
	b = input.b; % [mm]
	t = input.t; % [mm]
else
	b = 2.5*Lf(1, 1); % [mm]
	t = 2.5*Lf(1, 1); % [mm]
end
h = 3 * Lf(1, 1) + (numCrack-1)*Lf(1, 1)*1.5; % [mm]
	
results = struct('displacement', {}, 'stress', {}, 'Geo', {});

%%                                               Processing
%                                               Parallelization
if numCor > maxNumCompThreads
	numCor = maxNumCompThreads;
end
SetParallel(numCor);

%                                               Generating fibers
set(textControl, 'string', 'Generating of fibers: geometry'); pause(0.5);
% FibGeo - geometry of fibers
switch typeGen
	case 0
% 		fprintf('Load\n');
		FibGeo = load(FileName);
	case 1
% 		fprintf('Cast\n');
		FibGeo = GenFibNoSaw(b, t, h, Lf, df, Vff);
	case 2
% 		fprintf('Saw\n');
		FibGeo = GenFibSaw(b, t, h, Lf, df, Vff);
	otherwise
		fprintf('Incorrect parametr\n');
end

% results(1).Geo = FibGeo;

%                                               Generating fiber stochastic material and interface parameters
set(textControl, 'string', 'Generating of fibers: parameters'); pause(0.5);
% FibMat - parameters of fibers
% FibInt - parameters of interface
FibMat = GenMat(FibGeo, df, Ef, ftf);
FibInt = GenInt(FibGeo, tau0, Gd, beta);

for i = 1:numCrack

%                                               Selection of fiber
set(textControl, 'string', strcat('Selection of fibers, crack: ', sprintf('%02d',i))); pause(0.5);
% Zr - position of the crack
% FibCrackS - parameters on short embedded side
% FibCrackL - parameters on long embedded side
% FibCrackIn - parameters - pulling and rupture
% FibCrackPar - material parmeters

Zr = 1.5 * Lf(1, 1) + (i-1)*Lf(1, 1)*1.5;
[FibCrackS, FibCrackL, FibCrackIn, FibCrackPar] = SelFib(FibGeo, FibMat, FibInt, eta, Zr);

%                                               Calculation
set(textControl, 'string', strcat('Calculation of cohesive relationship, crack: ', sprintf('%02d',i))); pause(0.5);
% [cohesion, FibCrackIn] = Compute(typeCalc, FibCrackS, FibCrackL, FibCrackIn, FibCrackPar, eta, sc, src, dc, cc);
switch typeCalc
	case 0
% 		fprintf('SW\n');
		[cohesion, FibCrackIn] = CompSW(FibCrackS, FibCrackIn, FibCrackPar, eta, sc, src, dc);
	case 1
% 		fprintf('TW\n');
		[cohesion, FibCrackIn] = CompTW(FibCrackS, FibCrackL, FibCrackIn, FibCrackPar, eta, sc, src, dc, cc);
	otherwise
		fprintf('Incorrect parametr\n');
end

cohesion(:,2) = cohesion(:,2) / (b * t); % Conversion force to stress
results(i).displacement = cohesion(:,1);
results(i).stress = cohesion(:,2);
end
set(textControl, 'string', '')
end
